module breakout (clk, reset, en, l, r, ballX, ballY, paddleX, bricks);

input clk, reset, l, r, en;
output [9:0] ballX, paddleX;
output [8:0] ballY;
output [59:0] bricks;

reg [9:0] ballX, paddleX;
reg [8:0] ballY;
reg [59:0] bricks, newBricks;
reg [5:0] bricknum, bricknumOld;
reg inBrick;
reg [1:0] balldir, tmpDir;

wire [9:0] tmpXa, tmpXb;
wire [8:0] tmpYa, tmpYb;
wire hittingBrick;
wire hitBrick;

assign hitBrick = (bricknum == 0) ? 0 : 1;
assign hittingBrick = (bricknum == 0 || bricknum > 60) ? hitBrick : bricks[bricknum-1];
 
inbrick moduleE(ballX, ballY, bricknumOld);
inbrick moduleA(tmpXa, tmpYa, bricknum);
nextBallPos moduleB(balldir, ballX, ballY, tmpXa, tmpYa);
nextBallPos moduleC(tmpDir, ballX, ballY, tmpXb, tmpYb);
nextBallDir moduleD(balldir, bricknumOld, bricknum, ballX, ballY, tmpDir);

// calculate the paddle's next position
always @(posedge clk) begin
  if (en) begin
    if (reset)
      paddleX <= 320;
    else if (l)
      paddleX <= paddleX - 2;
    else if (r)
      paddleX <= paddleX + 2;
  end
end

// calculate the ball's next position, and the bricks
always @(posedge clk) begin
  if (reset) begin
    ballX <= 100;
    ballY <= 400;
    balldir <= 0;
    bricks <= 60'hFFFFFFFFFFFFFFF;
  end else if (en) begin   
    if (hittingBrick & hitBrick)
      bricks[bricknum-1] <= 0; 
    if (hittingBrick) begin    
      ballX <= tmpXb;
      ballY <= tmpYb;
      balldir <= tmpDir;
    end else begin
      ballX <= tmpXa;
      ballY <= tmpYa;
    end
  end    
end

endmodule



module inbrick(bx, by, bricknum);

input [9:0] bx;
input [8:0] by;
output [5:0] bricknum;

wire [7:0] bnum;
wire [7:0] ynum;
wire [5:0] brnum, bricknum;

assign ynum = by[8:5];

assign bnum = (ynum >= 3 && ynum <= 8) ? 10 * ynum + bx[9:6] - 29 : 0;
assign brnum = (bnum >= 60 || bnum==0) ? 0 : bnum;
assign bricknum = (bx > 638 || bx < 2 || by > 478 || by < 2) ? 62 : bnum;

endmodule



module nextBallPos(bdir, bx, by, outx, outy);

input [1:0] bdir;
input [9:0] bx;
input [8:0] by;
output [9:0] outx;
output [8:0] outy;

assign outx = (bdir[1]) ? (bx + 2) : (bx - 2);
assign outy = (bdir[0]) ? (by + 2) : (by - 2);

endmodule


module nextBallDir(bdir, bnumo, bnumn, bx, by, outdir);

input [1:0] bdir;
input [5:0] bnumo, bnumn;
input [9:0] bx;
input [8:0] by;
output [1:0] outdir;

assign outdir[1] = ((bnumn == 62 && (bx > 638 || bx < 2)) || (((bnumn - bnumo) != 0) && ((bnumn - bnumo) != -10) && ((bnumn - bnumo) != 10))) ? ~outdir[1] : outdir[1];
assign outdir[0] = ((bnumn == 62 && (by > 478 || by < 2)) || ((bnumn - bnumo) < -1) || ((bnumn - bnumo) > 1)) ? ~outdir[0] : outdir[0];

//assign outdir[1] = (bx > 638 || bx < 2 || xpos == 0 || xpos == 1 || xpos == 2 || xpos == 30 || xpos == 31) ? ~bdir[1] : bdir[1];
//assign outdir[0] = (by > 478 || by < 2 || ypos == 0 || ypos == 1 || ypos == 2 || ypos == 14 || ypos == 15) ? ~bdir[0] : bdir[0];

endmodule
