module PADCOLLIDE (clk, X, Y, VX, VY, OLDX, 
                        PAD1X, PAD1Y, PAD1VX, PAD1VY,
                        PAD2X, PAD2Y, PAD2VX, PAD2VY,  
                        NEWVX, NEWVY, XOUT, YOUT, COLLIDEONE, COLLIDETWO, SPLIT) ;

input clk ;
input [9:0] X ;
input [8:0] Y ;
input [3:0] VX ;
input [3:0] VY ;
input [9:0] OLDX;

input [9:0] PAD1X ;
input [8:0] PAD1Y ;
input [2:0] PAD1VX ;
input [3:0] PAD1VY ;
input [9:0] PAD2X ;
input [8:0] PAD2Y ;
input [2:0] PAD2VX ;
input [3:0] PAD2VY ;
output [3:0] NEWVX ;
output [3:0] NEWVY ;
output COLLIDEONE ;
output COLLIDETWO ;
output [9:0] XOUT ;
output [8:0] YOUT ;
output SPLIT;
reg [3:0] _NEWVX;
reg [3:0] _NEWVY;
reg _COLLIDEONE;
reg _COLLIDETWO;

assign NEWVX = _NEWVX;
assign NEWVY = _NEWVY;
assign COLLIDEONE = _COLLIDEONE;
assign COLLIDETWO = _COLLIDETWO;

reg [9:0] _XOUT;
reg [8:0] _YOUT;
assign XOUT = _XOUT;
assign YOUT = _YOUT;
reg _SPLIT;
assign SPLIT = _SPLIT;

//pad is 8 pixels wide, 32 pixels high, 
//bullet is 8x8 pixels
//collision occurs if x +7 >= padx & x <=padx +7 & y +7>=pady & y <=pady+7

always @(posedge clk) begin
	_SPLIT <= 0;
	_YOUT <= Y;
	_NEWVX <= VX;
	_NEWVY <= VY;
	//collision with pad1
	if(X + 7 >= PAD1X & X <= PAD1X + 7 &
	   Y + 7 >= PAD1Y & Y <= PAD1Y + 31) begin
	   _COLLIDEONE <= 1;
	   _COLLIDETWO <= 0;	   //simple collision  (should be modified later)

		//compute left or right side of paddle
		if(OLDX > PAD1X) begin
			_XOUT <= PAD1X + 8;
			if(VX >=0 & VX < 5)  //bullet moving right
				_NEWVX <= VX +1;
			else if(VX[3]) begin//negative speed
				if(VX >4'b1011) //speed  > -5
					_NEWVX <= ~VX +2;
				else begin //split ball into 2
					_SPLIT <= 1;
					_NEWVX <= (~VX +1);// >> 1;
				end
			end
		end
		else begin
			_XOUT <= PAD1X -8;
			if(VX > 0 & VX <= 5) begin
				if(VX < 5)
					_NEWVX <= ~VX; //negate and increment
				else
					_NEWVX <= ~VX +1; //negate 
			end
			else if( VX != 4'b1011)
				_NEWVX <= VX -1;
		end

		//compute new yspeed
		if(VY!= 4'b0111  & (PAD1VY != 0 & !PAD1VY[3])) 
			_NEWVY <= VY +1;
		else if(VY != 4'b1001 & PAD1VY[3])
			_NEWVY <= VY-1;
		else 
			_NEWVY <= VY;

	end
	else if(X + 7 >= PAD2X & X <= PAD2X + 7 &
	   Y + 7 >= PAD2Y & Y <= PAD2Y + 31) begin
	   	_COLLIDEONE <= 0;
	   	_COLLIDETWO <= 1;
	   	//compute left or right side of paddle
		if(OLDX < PAD2X) begin
			_XOUT <= PAD2X - 8;
			if(VX > 0 & VX <= 5) begin  //bullet moving right
				if(VX < 5) 
					_NEWVX <= ~VX;
				else begin
					_NEWVX <= (~VX+1)>>1;
					_SPLIT <= 1;
				end	
			end
			else if(VX != 4'b1011)
				_NEWVX <= VX -1;
		end
		else begin
			_XOUT <= PAD2X + 8;
			if(VX >= 0 & VX <5)
				_NEWVX <= VX +1;
			else if(VX[3])
				_NEWVX <= ~VX +2;
		end
		//compute new yspeed
		if(VY!= 4'b0111  & (PAD2VY != 0 & !PAD2VY[3])) 
			_NEWVY <= VY +1;
		else if(VY != 4'b1001 & PAD2VY[3])
			_NEWVY <= VY-1;
		else 
			_NEWVY <= VY;
	end 
	else begin
	   	_COLLIDEONE <= 0;
	   	_COLLIDETWO <= 0;
		_NEWVX <= VX;
		_NEWVY <= VY;
		_XOUT <= X;
	end
end		

endmodule 



